import React, { MutableRefObject } from 'react';
import { App, Button } from 'antd';
import {
  ActionType,
  ModalForm,
  ProFormDependency,
  ProFormRadio,
  ProFormSelect,
  ProFormText,
} from '@ant-design/pro-components';
import { phoneList, userIdList, validCron, whitespaceForbidden } from '@/constants/rules';
import { addAppTask, updateAppTask } from '@/services/api/appTask';
import { conversationTypeEnum } from '@/constants/enums';
import { getAppRobotOptions } from '@/services/api/appRobot';
import { getAppSftpOptions } from '@/services/api/appSftp';

/** 添加修改应用任务组件 */
const UpdateAppTask: React.FC<{
  /** Table action 的引用，便于自定义触发 */
  actionRef: MutableRefObject<ActionType | undefined>;
  /** 应用编码 */
  appId: string;
  record?: any;
}> = ({ actionRef, appId, record }) => {
  const { message } = App.useApp();
  /** 提交数据 */
  const onFinish = async (values: any) => {
    if (
      conversationTypeEnum.SINGLE.toString() === values?.conversationType &&
      !values?.phones &&
      !values?.userIds
    ) {
      message.warning('手机号或用户id必须有一个不为空');
      return false;
    }
    if (record) {
      await updateAppTask({ ...values, appId, taskId: record?.taskId });
      message.success('修改成功');
      actionRef.current?.reload();
      return true;
    }
    await addAppTask({ ...values, appId });
    message.success('添加成功');
    actionRef.current?.reload();
    return true;
  };

  return (
    <ModalForm
      title={record ? '修改任务' : '添加任务'}
      onFinish={onFinish}
      trigger={
        <Button danger={record} type={record ? 'default' : 'primary'}>
          {record ? '修改' : '添加'}
        </Button>
      }
    >
      <ProFormText
        name="taskName"
        label="任务名称"
        initialValue={record?.taskName}
        rules={[
          {
            required: true,
          },
          whitespaceForbidden,
        ]}
      />
      <ProFormText name="description" label="任务描述" initialValue={record?.description} />
      <ProFormText
        name="cron"
        label="cron表达式"
        initialValue={record?.cron}
        rules={[
          {
            required: true,
          },
          validCron,
        ]}
      />
      <ProFormSelect
        name="robotCode"
        label="机器人"
        initialValue={record?.robotCode}
        showSearch
        rules={[{ required: true }]}
        debounceTime={1000}
        // @ts-ignore
        request={async ({ keyWords = '' }) => {
          const res = await getAppRobotOptions({ appId });
          return res?.data?.filter(({ value, label }) => {
            return value?.includes(keyWords) || label?.includes(keyWords);
          });
        }}
      />
      <ProFormRadio.Group
        name="conversationType"
        label="会话类型"
        initialValue={
          record?.conversationType ? record.conversationType.toString() : record?.conversationType
        }
        valueEnum={conversationTypeEnum.valueEnum}
        rules={[{ required: true }]}
      />
      <ProFormDependency name={['conversationType']}>
        {({ conversationType }) => {
          // 单聊表单项
          if (conversationTypeEnum.SINGLE.toString() === conversationType) {
            return (
              <>
                <ProFormText
                  name="phones"
                  label="接收机器人消息的手机号列表"
                  initialValue={record?.phones}
                  rules={[phoneList]}
                />
                <ProFormText
                  name="userIds"
                  label="接收机器人消息的userId列表"
                  initialValue={record?.userIds}
                  rules={[userIdList]}
                />
              </>
            );
          }

          // 群聊表单项
          if (conversationTypeEnum.GROUP.toString() === conversationType) {
            return (
              <ProFormText
                name="openConversationId"
                label="群id"
                initialValue={record?.openConversationId}
                rules={[
                  {
                    required: true,
                  },
                  whitespaceForbidden,
                ]}
              />
            );
          }

          return <></>;
        }}
      </ProFormDependency>
      <ProFormSelect
        name="sftpId"
        label="sftp"
        initialValue={record?.sftpId}
        showSearch
        rules={[{ required: true }]}
        debounceTime={1000}
        // @ts-ignore
        request={async ({ keyWords = '' }) => {
          const res = await getAppSftpOptions({ appId });
          return res?.data?.filter(({ value, label }) => {
            return value?.includes(keyWords) || label?.includes(keyWords);
          });
        }}
      />
      <ProFormText
        name="dataDir"
        label="数据文件夹（绝对路径）"
        initialValue={record?.dataDir}
        rules={[
          {
            required: true,
          },
          whitespaceForbidden,
        ]}
      />
      <ProFormText
        name="filePattern"
        label="文件名匹配正则表达式"
        initialValue={record?.filePattern}
        rules={[
          {
            required: true,
          },
        ]}
      />
    </ModalForm>
  );
};

export default UpdateAppTask;
