import { ActionType, ProColumns, ProTable } from '@ant-design/pro-components';
import React, { useRef, useState } from 'react';
import { Button, Modal, Space, Table } from 'antd';
import { listAppRobot } from '@/services/api/appRobot';
import DeleteAppRobot from '@/pages/AppManage/components/DeleteAppRobot';
import AvailableRobot from '@/pages/AppManage/components/AvailableRobot';
import AppRobotCommandManage from '@/pages/AppManage/components/AppRobotCommandManage';

type TableListItem = API.ListAppRobotVo;

/** 应用机器人管理组件 */
const AppRobotManage: React.FC<{
  /** 受控的打开关闭 */
  open: boolean;
  /** 打开关闭的事件 */
  setOpen: (v: boolean) => void;
  /** 应用编码 */
  appId: string;
}> = ({ open, setOpen, appId }) => {
  // Table action 的引用，便于自定义触发
  const actionRef = useRef<ActionType>();
  const [isAvailableRobotOpen, setIsAvailableRobotOpen] = useState(false);
  const [isAppRobotCommandManageOpen, setIsAppRobotCommandManageOpen] = useState(false);
  const [robotCode, setRobotCode] = useState('');

  /**
   * 获取数据
   * @param params 参数
   * @param sort 排序
   */
  const getDataSource = async (params: any, sort: any) => {
    const res = await listAppRobot({
      ...params,
      sort,
      appId,
    });
    return Promise.resolve({
      success: true,
      data: res?.data?.list,
      total: res?.data?.total,
    });
  };

  // 表格列的配置描述
  const columns: ProColumns<TableListItem>[] = [
    {
      title: '机器人编码',
      dataIndex: 'robotCode',
      copyable: true,
      sorter: true,
    },
    {
      title: '机器人名称',
      dataIndex: 'robotName',
      copyable: true,
      sorter: true,
    },
    {
      title: '机器人密钥',
      dataIndex: 'robotSecret',
      copyable: true,
      search: false,
      sorter: true,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (text, record) => (
        <Space>
          <Button
            type="primary"
            onClick={() => {
              setRobotCode(record?.robotCode as string);
              setIsAppRobotCommandManageOpen(true);
            }}
          >
            命令管理
          </Button>
          <DeleteAppRobot
            actionRef={actionRef}
            appId={appId}
            robotCodes={[record?.robotCode as string]}
          />
        </Space>
      ),
    },
  ];

  return (
    <Modal
      title="机器人管理"
      open={open}
      onCancel={() => setOpen(false)}
      footer={false}
      destroyOnClose
      width={'100%'}
    >
      <ProTable<TableListItem>
        headerTitle="机器人列表"
        rowKey="robotCode"
        actionRef={actionRef}
        columns={columns}
        pagination={{
          showQuickJumper: true,
          defaultPageSize: 10,
        }}
        search={{
          layout: 'vertical',
        }}
        toolBarRender={() => [
          <Button key={'add'} type="primary" onClick={() => setIsAvailableRobotOpen(true)}>
            添加
          </Button>,
          <AvailableRobot
            key="available"
            open={isAvailableRobotOpen}
            setOpen={setIsAvailableRobotOpen}
            parentActionRef={actionRef}
            appId={appId}
          />,
        ]}
        request={getDataSource}
        rowSelection={{
          selections: [Table.SELECTION_ALL, Table.SELECTION_INVERT, Table.SELECTION_NONE],
        }}
        tableAlertRender={({ selectedRowKeys }) => <span>已选 {selectedRowKeys.length} 项</span>}
        tableAlertOptionRender={({ selectedRowKeys }) => (
          <DeleteAppRobot
            key="delete"
            actionRef={actionRef}
            appId={appId}
            robotCodes={selectedRowKeys as string[]}
          />
        )}
      />
      <AppRobotCommandManage
        open={isAppRobotCommandManageOpen}
        setOpen={setIsAppRobotCommandManageOpen}
        appId={appId}
        robotCode={robotCode}
      />
    </Modal>
  );
};

export default AppRobotManage;
