import { ActionType, ProColumns, ProTable } from '@ant-design/pro-components';
import React, { useRef, useState } from 'react';
import { App, Button, Modal, Space } from 'antd';
import { deleteAppSftp, listAppSftp, updateAppSftp } from '@/services/api/appSftp';
import { isEqual } from 'lodash-es';
import AddAppSftp from '@/pages/AppManage/components/AddAppSftp';
import { whitespaceForbidden } from '@/constants/rules';

type TableListItem = API.ListAppSftpVo;

/** 应用机器人管理组件 */
const AppSftpManage: React.FC<{
  /** 受控的打开关闭 */
  open: boolean;
  /** 打开关闭的事件 */
  setOpen: (v: boolean) => void;
  /** 应用编码 */
  appId: string;
}> = ({ open, setOpen, appId }) => {
  // Table action 的引用，便于自定义触发
  const actionRef = useRef<ActionType>();
  // 正在编辑的行
  const [editableKeys, setEditableRowKeys] = useState<React.Key[]>([]);
  const { message } = App.useApp();

  /**
   * 获取数据
   * @param params 参数
   * @param sort 排序
   */
  const getDataSource = async (params: any, sort: any) => {
    const res = await listAppSftp({
      ...params,
      sort,
      appId,
    });
    return Promise.resolve({
      success: true,
      data: res?.data?.list,
      total: res?.data?.total,
    });
  };

  // 表格列的配置描述
  const columns: ProColumns<TableListItem>[] = [
    {
      title: 'sftpId',
      dataIndex: 'sftpId',
      copyable: true,
      sorter: true,
      editable: false,
    },
    {
      title: 'ip',
      dataIndex: 'host',
      copyable: true,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: '端口',
      dataIndex: 'port',
      valueType: 'digit',
      copyable: true,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          {
            min: 0,
            type: 'number',
          },
          {
            max: 65535,
            type: 'number',
          },
        ],
      },
    },
    {
      title: '用户名',
      dataIndex: 'username',
      copyable: true,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: '密码',
      dataIndex: 'password',
      copyable: true,
      search: false,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '更新时间',
      dataIndex: 'updateTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (text, record, index, action) => (
        <Space>
          <Button
            danger
            onClick={() => {
              action?.startEditable?.(record?.sftpId as string);
            }}
          >
            编辑
          </Button>
        </Space>
      ),
    },
  ];

  /**
   * 行保存
   * @param key 行 id，一般是唯一id
   * @param record 当前修改的行的值，只有 form 在内的会被设置
   * @param originRow 原始值，可以用于判断是否修改
   */
  const onSave = async (key: any, record: any, originRow: any) => {
    if (isEqual(record, originRow)) {
      return Promise.reject();
    }

    await updateAppSftp(record);
    message.success('修改成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  /**
   * 行删除
   * @param key 行 id，一般是唯一id
   */
  const onDelete = async (key: any) => {
    await deleteAppSftp({ sftpId: key });
    message.success('删除成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  return (
    <Modal
      title="sftp管理"
      open={open}
      onCancel={() => setOpen(false)}
      footer={false}
      destroyOnClose
      width={'100%'}
    >
      <ProTable<TableListItem>
        headerTitle="sftp列表"
        rowKey="sftpId"
        actionRef={actionRef}
        columns={columns}
        pagination={{
          showQuickJumper: true,
          defaultPageSize: 10,
        }}
        search={{
          layout: 'vertical',
        }}
        toolBarRender={() => [<AddAppSftp key={'add'} actionRef={actionRef} appId={appId} />]}
        request={getDataSource}
        editable={{
          type: 'single',
          editableKeys,
          onChange: setEditableRowKeys,
          onSave,
          onDelete,
        }}
      />
    </Modal>
  );
};

export default AppSftpManage;
