import { ActionType, ProColumns, ProTable } from '@ant-design/pro-components';
import React, { useRef } from 'react';
import { Modal, Space } from 'antd';
import { listAppTask } from '@/services/api/appTask';
import { appTaskStatusEnum, conversationTypeEnum } from '@/constants/enums';
import UpdateAppTask from '@/pages/AppManage/components/AddAppTask';
import StopAppTask from '@/pages/AppManage/components/StopAppTask';
import EnableAppTask from '@/pages/AppManage/components/EnableAppTask';
import DeleteAppTask from '@/pages/AppManage/components/DeleteAppTask';
import RunAppTask from '@/pages/AppManage/components/RunAppTask';
import AppTaskLogModal from '@/pages/AppManage/components/AppTaskLogModal';

type TableListItem = API.ListAppTaskVo;

/** 应用任务管理组件 */
const AppTaskManage: React.FC<{
  /** 受控的打开关闭 */
  open: boolean;
  /** 打开关闭的事件 */
  setOpen: (v: boolean) => void;
  /** 应用编码 */
  appId: string;
}> = ({ open, setOpen, appId }) => {
  // Table action 的引用，便于自定义触发
  const actionRef = useRef<ActionType>();

  /**
   * 获取数据
   * @param params 参数
   * @param sort 排序
   */
  const getDataSource = async (params: any, sort: any) => {
    const res = await listAppTask({
      ...params,
      sort,
      appId,
    });
    return Promise.resolve({
      success: true,
      data: res?.data?.list,
      total: res?.data?.total,
    });
  };

  // 表格列的配置描述
  const columns: ProColumns<TableListItem>[] = [
    {
      title: '任务id',
      dataIndex: 'taskId',
      copyable: true,
      sorter: true,
    },
    {
      title: '任务名称',
      dataIndex: 'taskName',
      copyable: true,
      sorter: true,
    },
    {
      title: '任务描述',
      dataIndex: 'description',
      copyable: true,
      search: false,
      ellipsis: true,
    },
    {
      title: '任务状态',
      dataIndex: 'status',
      valueEnum: appTaskStatusEnum.valueEnum,
      sorter: true,
    },
    {
      title: 'cron表达式',
      dataIndex: 'cron',
      copyable: true,
      search: false,
    },
    {
      title: '机器人编码',
      dataIndex: 'robotCode',
      copyable: true,
      sorter: true,
    },
    {
      title: '机器人名称',
      dataIndex: 'robotName',
      copyable: true,
      sorter: true,
    },
    {
      title: '会话类型',
      dataIndex: 'conversationType',
      valueEnum: conversationTypeEnum.valueEnum,
      sorter: true,
    },
    {
      title: '群id',
      dataIndex: 'openConversationId',
      copyable: true,
      sorter: true,
    },
    {
      title: '接收机器人消息的手机号列表',
      dataIndex: 'phones',
      copyable: true,
      sorter: true,
    },
    {
      title: '接收机器人消息的userId列表',
      dataIndex: 'userIds',
      copyable: true,
      sorter: true,
    },
    {
      title: 'sftpId',
      dataIndex: 'sftpId',
      copyable: true,
      sorter: true,
    },
    {
      title: 'sftp ip',
      dataIndex: 'host',
      copyable: true,
      sorter: true,
    },
    {
      title: '数据文件夹',
      dataIndex: 'dataDir',
      copyable: true,
      search: false,
    },
    {
      title: '文件名匹配正则表达式',
      dataIndex: 'filePattern',
      copyable: true,
      search: false,
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
    },
    {
      title: '更新时间',
      dataIndex: 'updateTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (text, record) => (
        <Space direction="vertical">
          <UpdateAppTask actionRef={actionRef} appId={appId} record={record} />
          {appTaskStatusEnum.STOPPED === record?.status ? (
            <EnableAppTask actionRef={actionRef} taskId={record?.taskId as string} />
          ) : (
            <StopAppTask actionRef={actionRef} taskId={record?.taskId as string} />
          )}
          <DeleteAppTask actionRef={actionRef} taskId={record?.taskId as string} />
          <RunAppTask actionRef={actionRef} taskId={record?.taskId as string} />
          <AppTaskLogModal taskId={record?.taskId} />
        </Space>
      ),
    },
  ];

  return (
    <Modal
      title="任务管理"
      open={open}
      onCancel={() => setOpen(false)}
      footer={false}
      destroyOnClose
      width={'100%'}
    >
      <ProTable<TableListItem>
        headerTitle="任务列表"
        rowKey="taskId"
        actionRef={actionRef}
        columns={columns}
        pagination={{
          showQuickJumper: true,
          defaultPageSize: 10,
        }}
        search={{
          layout: 'vertical',
        }}
        toolBarRender={() => [<UpdateAppTask key={'add'} actionRef={actionRef} appId={appId} />]}
        request={getDataSource}
      />
    </Modal>
  );
};

export default AppTaskManage;
