import { ActionType, ProColumns, ProTable } from '@ant-design/pro-components';
import React, { useRef, useState } from 'react';
import { App, Button, Popconfirm, Space } from 'antd';
import { isEqual } from 'lodash-es';
import AddApp from '@/pages/AppManage/components/AddApp';
import { deleteApp, listApp, resetAppSecret, updateApp } from '@/services/api/app';
import { whitespaceForbidden } from '@/constants/rules';
import AppRobotManage from '@/pages/AppManage/components/AppRobotManage';
import AppSftpManage from '@/pages/AppManage/components/AppSftpManage';
import AppTaskManage from '@/pages/AppManage/components/AppTaskManage';

type TableListItem = API.ListAppVo;

/**
 * 获取数据
 * @param params 参数
 * @param sort 排序
 */
const getDataSource = async (params: any, sort: any) => {
  const res = await listApp({
    ...params,
    sort,
  });
  return Promise.resolve({
    success: true,
    data: res?.data?.list,
    total: res?.data?.total,
  });
};

/** 应用管理页面 */
export default () => {
  // Table action 的引用，便于自定义触发
  const actionRef = useRef<ActionType>();
  // 正在编辑的行
  const [editableKeys, setEditableRowKeys] = useState<React.Key[]>([]);
  const { message } = App.useApp();
  const [isAppRobotManageOpen, setIsAppRobotManageOpen] = useState(false);
  const [isAppSftpManageOpen, setIsAppSftpManageOpen] = useState(false);
  const [isAppTaskManageOpen, setIsAppTaskManageOpen] = useState(false);
  const [appId, setAppId] = useState('');

  /**
   * 重置appId
   * @param appId 应用编码
   */
  const updateAppSecret = async (appId: string) => {
    try {
      await resetAppSecret({ appId });
      message.success('操作成功');
      actionRef.current?.reload();
      return Promise.resolve();
    } catch (e) {
      return Promise.reject();
    }
  };

  // 表格列的配置描述
  const columns: ProColumns<TableListItem>[] = [
    {
      title: '应用名称',
      dataIndex: 'appName',
      copyable: true,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: 'appId',
      dataIndex: 'appId',
      copyable: true,
      sorter: true,
      editable: false,
    },
    {
      title: 'appSecret',
      dataIndex: 'appSecret',
      copyable: true,
      sorter: true,
      search: false,
      editable: false,
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '更新时间',
      dataIndex: 'updateTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (text, record, index, action) => (
        <Space>
          <Button
            danger
            onClick={() => {
              action?.startEditable?.(record?.appId as string);
            }}
          >
            编辑
          </Button>
          <Popconfirm title="确定？" onConfirm={() => updateAppSecret(record?.appId as string)}>
            <Button danger>重置appSecret</Button>
          </Popconfirm>
          <Button
            type="primary"
            onClick={() => {
              setAppId(record?.appId as string);
              setIsAppRobotManageOpen(true);
            }}
          >
            机器人管理
          </Button>
          <Button
            type="primary"
            onClick={() => {
              setAppId(record?.appId as string);
              setIsAppSftpManageOpen(true);
            }}
          >
            sftp管理
          </Button>
          <Button
            type="primary"
            onClick={() => {
              setAppId(record?.appId as string);
              setIsAppTaskManageOpen(true);
            }}
          >
            任务管理
          </Button>
        </Space>
      ),
    },
  ];

  /**
   * 行保存
   * @param key 行 id，一般是唯一id
   * @param record 当前修改的行的值，只有 form 在内的会被设置
   * @param originRow 原始值，可以用于判断是否修改
   */
  const onSave = async (key: any, record: any, originRow: any) => {
    if (isEqual(record, originRow)) {
      return Promise.reject();
    }

    await updateApp(record);
    message.success('修改成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  /**
   * 行删除
   * @param key 行 id，一般是唯一id
   */
  const onDelete = async (key: any) => {
    await deleteApp({ appId: key });
    message.success('删除成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  return (
    <>
      <ProTable<TableListItem>
        headerTitle="应用列表"
        rowKey="appId"
        actionRef={actionRef}
        columns={columns}
        pagination={{
          showQuickJumper: true,
          defaultPageSize: 10,
        }}
        search={{
          layout: 'vertical',
        }}
        toolBarRender={() => [<AddApp key="add" actionRef={actionRef} />]}
        request={getDataSource}
        editable={{
          type: 'single',
          editableKeys,
          onChange: setEditableRowKeys,
          onSave,
          onDelete,
        }}
      />
      <AppRobotManage open={isAppRobotManageOpen} setOpen={setIsAppRobotManageOpen} appId={appId} />
      <AppSftpManage open={isAppSftpManageOpen} setOpen={setIsAppSftpManageOpen} appId={appId} />
      <AppTaskManage open={isAppTaskManageOpen} setOpen={setIsAppTaskManageOpen} appId={appId} />
    </>
  );
};
