import { ActionType, ProColumns, ProTable } from '@ant-design/pro-components';
import React, { useRef, useState } from 'react';
import { Button, message } from 'antd';
import { isEqual } from 'lodash-es';
import AddRobot from '@/pages/RobotManage/components/AddRobot';
import { deleteRobot, listRobot, updateRobot } from '@/services/api/robot';
import { whitespaceForbidden } from '@/constants/rules';

type TableListItem = API.ListRobotVo;

/**
 * 获取数据
 * @param params 参数
 * @param sort 排序
 */
const getDataSource = async (params: any, sort: any) => {
  const res = await listRobot({
    ...params,
    sort,
  });
  return Promise.resolve({
    success: true,
    data: res?.data?.list,
    total: res?.data?.total,
  });
};

/** 机器人管理页面 */
export default () => {
  // Table action 的引用，便于自定义触发
  const actionRef = useRef<ActionType>();
  // 正在编辑的行
  const [editableKeys, setEditableRowKeys] = useState<React.Key[]>([]);

  // 表格列的配置描述
  const columns: ProColumns<TableListItem>[] = [
    {
      title: '机器人编码',
      dataIndex: 'robotCode',
      copyable: true,
      sorter: true,
      editable: false,
    },
    {
      title: '机器人名称',
      dataIndex: 'robotName',
      copyable: true,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: '机器人密钥',
      dataIndex: 'robotSecret',
      copyable: true,
      search: false,
      sorter: true,
      formItemProps: {
        rules: [
          {
            required: true,
          },
          whitespaceForbidden,
        ],
      },
    },
    {
      title: '创建时间',
      dataIndex: 'createTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '更新时间',
      dataIndex: 'updateTime',
      valueType: 'dateTime',
      search: false,
      sorter: true,
      editable: false,
    },
    {
      title: '操作',
      key: 'option',
      valueType: 'option',
      render: (text, record, index, action) => (
        <Button
          danger
          onClick={() => {
            action?.startEditable?.(record.robotCode as string);
          }}
        >
          编辑
        </Button>
      ),
    },
  ];

  /**
   * 行保存
   * @param key 行 id，一般是唯一id
   * @param record 当前修改的行的值，只有 form 在内的会被设置
   * @param originRow 原始值，可以用于判断是否修改
   */
  const onSave = async (key: any, record: any, originRow: any) => {
    if (isEqual(record, originRow)) {
      return Promise.reject();
    }

    await updateRobot(record);
    message.success('修改成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  /**
   * 行删除
   * @param key 行 id，一般是唯一id
   */
  const onDelete = async (key: any) => {
    await deleteRobot({ robotCode: key });
    message.success('删除成功');
    actionRef.current?.reload();
    return Promise.resolve();
  };

  return (
    <ProTable<TableListItem>
      headerTitle="机器人列表"
      rowKey="robotCode"
      actionRef={actionRef}
      columns={columns}
      pagination={{
        showQuickJumper: true,
        defaultPageSize: 10,
      }}
      search={{
        layout: 'vertical',
      }}
      toolBarRender={() => [<AddRobot key="add" actionRef={actionRef} />]}
      request={getDataSource}
      editable={{
        type: 'single',
        editableKeys,
        onChange: setEditableRowKeys,
        onSave,
        onDelete,
      }}
    />
  );
};
